/*
 * (C) Copyright IBM Corporation 1999, 2016.
 * (C) Copyright HCL Technologies Limited 2016, 2017.
*/
/*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA

 Author: IBM Corporation and/or HCL Technologies Ltd.
 This module is part of the IBM (R) Rational (R) ClearCase (R)(*)
 Multi-version file system (MVFS).
 For support, please visit http://www.ibm.com/software/support

 * ClearCase is a trademark of IBM Corporation, registered in many
   jurisdictions, and is used under license.
*/
#include "vnode_linux.h"

/* Address space operations for MVFS */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,7,0) || \
    (defined(RATL_SUSE) && (RATL_VENDOR_VER >= 1203))

static ssize_t mvfs_direct_IO(
    struct kiocb *iocb,
    struct iov_iter *iter
);
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(3,16,0) /* 3.16.0 later but before 4.7.0 */
static ssize_t mvfs_direct_IO(
    struct kiocb *iocb,
    struct iov_iter *iter,
    loff_t offset
);
#else /* !LINUX_VERSION_CODE >= KERNEL_VERSION(3,16,0)*/
static ssize_t
mvfs_direct_IO(
    int rw,
    struct kiocb *iocb,
    const struct iovec *iov,
    loff_t pos,
    unsigned long nr_segs
);
#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(4,7,0) */

/* We used to have stub mapping operations.  All they would do would be
 * to report an error.  However, even the existence of these functions
 * would make sendfile think that it would work on our files.  By not
 * having the functions declared at all, we cause it to find out sooner.
 * However, direct_IO must be implemented, otherwise open with O_DIRECT fails.
 */

struct address_space_operations mvfs_addrspace_ops = {
    .direct_IO = mvfs_direct_IO
};

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,7,0) || \
    (defined(RATL_SUSE) && (RATL_VENDOR_VER >= 1203))
ssize_t mvfs_direct_IO(
    struct kiocb *iocb,
    struct iov_iter *iter
)
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(3,16,0) /* 3.16.0 later but before 4.7.0 */
ssize_t mvfs_direct_IO(
    struct kiocb *iocb,
    struct iov_iter *iter,
    loff_t offset
)
#else /* !LINUX_VERSION_CODE >= KERNEL_VERSION(3,16,0)*/
ssize_t
mvfs_direct_IO(
    int rw,
    struct kiocb *iocb,
    const struct iovec *iov,
    loff_t pos,
    unsigned long nr_segs
)
#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(4,7,0) */
{
    /* return 0 cause it to fall back to buffered IO */
    return 0;
}
static const char vnode_verid_mvfs_linux_asops_c[] = "$Id:  08fc2978.c55f11e7.9b1a.00:01:84:fa:a2:18 $";
